@@ -12,7 +12,7 @@ Sigma<-as.vector(family$variance(Mu))
 if(overdispersion)
   Sigma<-Sigma*phi
 D<-as.vector(family$mu.eta(Eta))
-W0_inv<-D*1/Sigma*D
+#W0_inv<-D*1/Sigma*D
 
 if(print.iter.final)
 print(paste("Final Re-estimation Iteration ", 1,sep=""))
@@ -31,7 +31,10 @@ for(jf in 1:n)
 P1[(lin+dim.smooth+(jf-1)*s+1):(lin+dim.smooth+jf*s),(lin+dim.smooth+(jf-1)*s+1):(lin+dim.smooth+jf*s)]<-chol2inv(chol(q_start))
 }
 
+
 Delta<-matrix(0,steps,(lin+dim.smooth+s*n))
+Eta.ma<-matrix(0,steps+1,N)
+Eta.ma[1,]<-Eta
 
 Q<-list()
 Q[[1]]<-q_start
@@ -43,9 +46,15 @@ opt<-steps
 score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
 F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
 
+
+
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
 if(class(InvFisher)=="try-error")
-InvFisher<-solve(F_gross)  
+InvFisher<-try(solve(F_gross),silent=TRUE)  
+if(class(InvFisher)=="try-error")
+stop("Fisher matrix not invertible")
+
+
 
 half.index<-0
 solve.test<-FALSE
@@ -132,7 +141,6 @@ phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
 Sigma<-Sigma*phi
 }
 
-P1.old.temp<-P1
 
 if(s==1)
 {
@@ -150,10 +158,8 @@ score_vec<-t(Z_alles)%*%((y-Mu)*D*1/Sigma)-P1%*%Delta[1,]
 F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-
 if(class(InvFisher)=="try-error")
   InvFisher<-try(solve(F_gross),silent=TRUE)  
-
 if(class(InvFisher)=="try-error")
 {
   half.index<-half.index+1  
@@ -162,6 +168,9 @@ if(class(InvFisher)=="try-error")
 }
 }
 
+Eta.ma[2,]<-Eta
+
+P1.old.temp<-P1
 ###############################################################################################################################################
 ################################################################### Main Iteration ###################################################################
 eps<-eps.final*sqrt(length(Delta_r))
@@ -175,20 +184,23 @@ half.index<-0
 solve.test<-FALSE
 
 P1.old<-P1
-P1.old2<-P1.old.temp
 
 Delta_r<-InvFisher%*%score_vec
 ######### big while loop for testing if the update leads to Fisher matrix which can be inverted
+first.time<-FALSE
 while(!solve.test)
 {  
   
 solve.test2<-FALSE  
 while(!solve.test2)
 {  
-if(half.index>500)
+if(half.index>100)
 {
-half.index<-Inf;P1.old<-P1.old2
+half.index<-Inf;P1.old<-P1.old.temp
 }
+#D.old<-D
+#Sigma.old<-Sigma
+
 Delta[l,]<-Delta[l-1,]+nue*(0.5^half.index)*Delta_r
 Eta<-Z_alles%*%Delta[l,]
 Mu<-as.vector(family$linkinv(Eta))
@@ -198,25 +210,31 @@ D<-as.vector(family$mu.eta(Eta))
 if (method=="EM" || overdispersion)
 {  
   F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1.old
-  InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-  if(class(InvFisher)=="try-error")
-    InvFisher<-try(solve(F_gross),silent=TRUE)  
-  if(class(InvFisher)=="try-error")
+  InvFisher2<-try(chol2inv(chol(F_gross)),silent=TRUE)
+  if(class(InvFisher2)=="try-error")
+    InvFisher2<-try(solve(F_gross),silent=TRUE)  
+  if(class(InvFisher2)=="try-error")
   {
     half.index<-half.index+1  
   }else{
+    if(!first.time)
+    half.index.final<-half.index
     solve.test2<-TRUE 
+    first.time<-TRUE
   }}else{
-    solve.test2<-TRUE
+    if(!first.time)
+    half.index.final<-half.index
+    solve.test2<-TRUE 
+    first.time<-TRUE
   }}
 
 if (method=="EM")
 {
 ############################# Q update ################
-Q1<-InvFisher[(lin+dim.smooth+1):(lin+dim.smooth+s),(lin+dim.smooth+1):(lin+dim.smooth+s)]+Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)]%*%t(Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)])
+Q1<-InvFisher2[(lin+dim.smooth+1):(lin+dim.smooth+s),(lin+dim.smooth+1):(lin+dim.smooth+s)]+Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)]%*%t(Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)])
 
 for (i in 2:n)
-Q1<-Q1+InvFisher[(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s),(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]+Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]%*%t(Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)])
+Q1<-Q1+InvFisher2[(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s),(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]+Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]%*%t(Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)])
 
 Q1<-1/n*Q1
 }else{
@@ -263,7 +281,6 @@ phi<-(sum((y-Mu)^2/Mu))/(N-sum(diag(FinalHat)))
 Sigma<-Sigma*phi
 }
 
-P1.old.temp<-P1.old
 
 if(s==1)
 {
@@ -291,35 +308,142 @@ if(class(InvFisher)=="try-error")
 }
 }
 
-
-kritval<-sqrt(sum((Delta[l-1,]-Delta[l,])^2))/sqrt(sum(Delta[l-1,]^2))
-if(kritval<eps)
-break
+Eta.ma[l+1,]<-Eta
 
 
-if(l>2)
-{
-kritval2<-sqrt(sum((Delta[l-2,]-Delta[l,])^2))/sqrt(sum(Delta[l-2,]^2))
-if(kritval2<eps)
-break
-}
+P1.old.temp<-P1.old
+
+finish<-(sqrt(sum((Eta.ma[l,]-Eta.ma[l+1,])^2))/sqrt(sum((Eta.ma[l,])^2))<eps)
+finish2<-(sqrt(sum((Eta.ma[l-1,]-Eta.ma[l+1,])^2))/sqrt(sum((Eta.ma[l-1,])^2))<eps)
+
+if(finish ||  finish2) 
+  break
 }
 
+######## Final calculation
 
 opt<-l
+
+
+if(!solve.test)
+{
+  Delta[l,]<-Delta[l-1,]+nue*(0.5^half.index.final)*Delta_r
+  Eta<-Z_alles%*%Delta[l,]
+  Mu<-as.vector(family$linkinv(Eta))
+  Sigma<-as.vector(family$variance(Mu))
+  D<-as.vector(family$mu.eta(Eta))
+  
+  if (method=="EM" || overdispersion)
+  {  
+    F_gross<-t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1.old
+    InvFisher2<-try(chol2inv(chol(F_gross)),silent=TRUE)
+    if(class(InvFisher2)=="try-error")
+      InvFisher2<-try(solve(F_gross),silent=TRUE)  
+  }
+
+if (method=="EM")
+{
+  ############################# Q update ################
+  Q1<-InvFisher2[(lin+dim.smooth+1):(lin+dim.smooth+s),(lin+dim.smooth+1):(lin+dim.smooth+s)]+Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)]%*%t(Delta[l,(lin+dim.smooth+1):(lin+dim.smooth+s)])
+  
+  for (i in 2:n)
+    Q1<-Q1+InvFisher2[(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s),(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]+Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)]%*%t(Delta[l,(lin+dim.smooth+(i-1)*s+1):(lin+dim.smooth+i*s)])
+  
+  Q1<-1/n*Q1
+}else{
+  Eta_tilde<-Eta+(y-Mu)*1/D
+  
+  Betadach<-Delta[l,1:(lin+dim.smooth)]
+  
+  if(s==1)
+  {
+    upp<-max(upp,Q.fac*Q1)
+    low<-min(low,(1/Q.fac)*Q1)
+    optim.obj<-nlminb(sqrt(Q1),likelihood_nlminb,D=D,Sigma=Sigma,X=cbind(X,Phi),X_aktuell=cbind(X,Phi),Eta_tilde=Eta_tilde,n=n,Betadach=Betadach,W=W, lower = low, upper = upp)
+    Q1<-as.matrix(optim.obj$par)^2
+  }else{
+    up1<-max(up1,Q.fac*max(Q1))  
+    upp<-rep(up1,length(q_start_vec))
+    low<-c(rep(0,s),rep(-up1,0.5*(s^2-s)))
+    Q1_vec<-c(diag(Q1),Q1[lower.tri(Q1)])
+    optim.obj<-bobyqa(Q1_vec,likelihood,D=D,Sigma=Sigma,X=cbind(X,Phi),X_aktuell=cbind(X,Phi),Eta_tilde=Eta_tilde,n=n,s=s,k=k,Betadach=Betadach,W=W,lower=low,upper=upp)
+    
+    Q1<-matrix(0,s,s)
+    Q1[lower.tri(Q1)]<-optim.obj$par[(s+1):(s*(s+1)*0.5)]
+    Q1<-Q1+t(Q1)
+    diag(Q1)<-(optim.obj$par[1:s])
+    
+    #### Check for positiv definitness ########
+    for (ttt in 0:100)
+    {
+      Q1[lower.tri(Q1)]<-((0.5)^ttt)*Q1[lower.tri(Q1)]
+      Q1[upper.tri(Q1)]<-((0.5)^ttt)*Q1[upper.tri(Q1)]
+      Q_solvetest<-try(solve(Q1))
+      if(all (eigen(Q1)$values>0) & class(Q_solvetest)!="try-error")
+        break
+    }
+  }}
+
+Q[[l+1]]<-Q1
+
+
+  FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))
+  complexity<-sum(diag(FinalHat))
+
+if(overdispersion)
+{
+  phi<-(sum((y-Mu)^2/Mu))/(N-complexity)
+  Sigma<-Sigma*phi
+}
+}
+
+FinalHat<-(Z_alles*sqrt(Sigma*D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D*1/Sigma))
+complexity<-sum(diag(FinalHat))
+
 Deltafinal<-Delta[l,]
 Q_final<-Q[[l+1]]
 
 Standard_errors<-sqrt(diag(InvFisher))
 
-#FinalHat<-(Z_alles*sqrt(D*1/Sigma*D))%*%Inv_F_opt%*%t(Z_alles*sqrt(D*1/Sigma*D))
+
+#FinalHat<-(Z_alles*sqrt(D*1/Sigma*D))%*%InvFisher%*%t(Z_alles*sqrt(D*1/Sigma*D))
+
+#### compare Wood's Book on page 167
+if(s==1)
+{
+  P1a<-c(rep(0,lin+dim.smooth),rep((Q1^(-1)),n*s))
+  P1a<-diag(P1a)
+}else{
+  P1a<-matrix(0,lin+dim.smooth+n*s,lin+dim.smooth+n*s)
+  for(jf in 1:n)
+    P1a[(lin+dim.smooth+(jf-1)*s+1):(lin+dim.smooth+jf*s),
+        (lin+dim.smooth+(jf-1)*s+1):(lin+dim.smooth+jf*s)]<-chol2inv(chol(Q1))
+}
+#### compare Wood's Book on page 167
+EDF.matrix<-InvFisher%*%(t(Z_alles)%*%(Z_alles*D*1/Sigma*D)+P1a)
+
+
+## compute ranef part of loglik
+if(s==1)
+{
+  P1.ran<-rep((Q_final^(-1)),n*s)
+  P1.ran<-diag(P1.ran)
+}else{
+  P1.ran<-matrix(0,n*s,n*s)
+  for(jf in 1:n)
+    P1.ran[((jf-1)*s+1):(jf*s),((jf-1)*s+1):(jf*s)]<-chol2inv(chol(Q_final))
+}
+
+ranef.logLik<--0.5*t(Deltafinal[(lin+dim.smooth+1):(lin+dim.smooth+n*s)])%*%P1.ran%*%Deltafinal[(lin+dim.smooth+1):(lin+dim.smooth+n*s)]
 
 ret.obj=list()
+ret.obj$ranef.logLik<-ranef.logLik
 ret.obj$opt<-opt
 ret.obj$Delta<-Deltafinal
 ret.obj$Q<-Q_final
 ret.obj$Standard_errors<-Standard_errors
 ret.obj$phi<-phi
-#ret.obj$complexity<-sum(diag(FinalHat))
+ret.obj$EDF.matrix<-EDF.matrix
+ret.obj$complexity<-complexity
 return(ret.obj)
 }
